// Copyright (C) 2016 Kurt Pattyn <pattyn.kurt@gmail.com>.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QWEBSOCKETSGLOBAL_H
#define QWEBSOCKETSGLOBAL_H

#include <QtCore/qglobal.h>
#include <QtWebSockets/qtwebsocketsexports.h>

#endif // QWEBSOCKETSGLOBAL_H
